@@ -82,8 +82,6 @@ struct _ECalBackendGroupwisePrivate {
 	gchar *uri;
 	ECredentials *credentials;
 	gchar *container_id;
-	gboolean is_online;
-	gboolean is_online_changed;
 	GHashTable *categories_by_id;
 	GHashTable *categories_by_name;
 
@@ -453,7 +451,8 @@ get_deltas (gpointer handle)
 
 	cbgw = (ECalBackendGroupwise *) handle;
 	priv = cbgw->priv;
-	if (!priv->is_online)
+
+	if (!e_backend_get_online (E_BACKEND (cbgw)))
 		return FALSE;
 
 	kind = e_cal_backend_get_kind (E_CAL_BACKEND (cbgw));
@@ -758,7 +757,7 @@ get_cache_refresh_interval (ECalBackendGroupwise *cbgw)
 	ESource *source;
 
 	time_interval = CACHE_REFRESH_INTERVAL;
-	source = e_cal_backend_get_source (backend);
+	source = e_backend_get_source (E_BACKEND (backend));
 
 	time_interval_string = g_getenv ("GETQM_TIME_INTERVAL");
 
@@ -853,7 +852,7 @@ e_cal_backend_groupwise_refresh_calendar (ECalBackendGroupwise *cbgw)
 	ECalBackendGroupwisePrivate *priv = cbgw->priv;
 	gboolean delta_started = FALSE;
 
-	if (!priv->is_online)
+	if (!e_backend_get_online (E_BACKEND (cbgw)))
 		return;
 
 	PRIV_LOCK (priv);
@@ -926,8 +925,6 @@ cache_init (ECalBackendGroupwise *cbgw)
 		g_warning (G_STRLOC ": Could not get the categories from the server");
 	}
 
-	priv->is_online = TRUE;
-
 	/* We poke the cache for a default timezone. Its
 	 * absence indicates that the cache file has not been
 	 * populated before. */
@@ -969,6 +966,7 @@ set_container_id_with_count (ECalBackendGroupwise *cbgw,
 	GList *container_list = NULL, *l;
 	EGwConnectionStatus status;
 	icalcomponent_kind kind;
+	ESource *source;
 
 	priv = cbgw->priv;
 
@@ -978,7 +976,8 @@ set_container_id_with_count (ECalBackendGroupwise *cbgw,
 	case ICAL_VEVENT_COMPONENT:
 	case ICAL_VTODO_COMPONENT:
 	case ICAL_VJOURNAL_COMPONENT:
-		e_source_set_name (e_cal_backend_get_source (E_CAL_BACKEND (cbgw)), _("Calendar"));
+		source = e_backend_get_source (E_BACKEND (cbgw));
+		e_source_set_name (source, _("Calendar"));
 		break;
 	default:
 		priv->container_id = NULL;
@@ -1033,6 +1032,7 @@ connect_to_server (ECalBackendGroupwise *cbgw,
 	gchar *parent_user = NULL;
 	icalcomponent_kind kind;
 	EGwConnectionErrors errors;
+	gboolean online;
 
 	priv = cbgw->priv;
 
@@ -1041,7 +1041,7 @@ connect_to_server (ECalBackendGroupwise *cbgw,
 		return;
 	}
 
-	source = e_cal_backend_get_source (E_CAL_BACKEND (cbgw));
+	source = e_backend_get_source (E_BACKEND (cbgw));
 	real_uri = NULL;
 	if (source)
 		real_uri = form_uri (source);
@@ -1106,13 +1106,13 @@ connect_to_server (ECalBackendGroupwise *cbgw,
 	g_free (real_uri);
 
 	e_cal_backend_notify_readonly (E_CAL_BACKEND (cbgw), cbgw->priv->read_only);
-	e_cal_backend_notify_online (E_CAL_BACKEND (cbgw), priv->is_online);
+
+	online = e_backend_get_online (E_BACKEND (cbgw));
+	e_cal_backend_notify_online (E_CAL_BACKEND (cbgw), online);
 
 	if (priv->cnc ) {
 		if (priv->store && priv->container_id) {
-			priv->is_online = TRUE;
-			if (priv->is_online_changed && !priv->dthread) {
-				priv->is_online_changed = FALSE;
+			if (!priv->dthread) {
 				fetch_deltas (cbgw);
 			}
 
@@ -1132,7 +1132,6 @@ connect_to_server (ECalBackendGroupwise *cbgw,
 			g_free (errors.description);
 		return;
 	}
-	priv->is_online_changed = FALSE;
 
 	if (E_IS_GW_CONNECTION (priv->cnc)) {
 		ECalBackend *backend;
@@ -1291,7 +1290,7 @@ e_cal_backend_groupwise_get_backend_property (ECalBackendSync *backend,
 		cbgw = E_CAL_BACKEND_GROUPWISE (backend);
 		priv = cbgw->priv;
 
-		if (priv->is_online) {
+		if (e_backend_get_online (E_BACKEND (cbgw))) {
 			if (priv->user_email)
 				g_free (priv->user_email);
 
@@ -1370,6 +1369,7 @@ e_cal_backend_groupwise_open (ECalBackendSync *backend,
 	ECalBackendGroupwise *cbgw;
 	ECalBackendGroupwisePrivate *priv;
 	const gchar *cache_dir;
+	gboolean online;
 
 	cbgw = E_CAL_BACKEND_GROUPWISE (backend);
 	priv = cbgw->priv;
@@ -1379,16 +1379,18 @@ e_cal_backend_groupwise_open (ECalBackendSync *backend,
 	PRIV_LOCK (priv);
 
 	cbgw->priv->read_only = FALSE;
-	e_cal_backend_notify_online (E_CAL_BACKEND (backend), priv->is_online);
 
-	if (!priv->is_online) {
+	online = e_backend_get_online (E_BACKEND (backend));
+	e_cal_backend_notify_online (E_CAL_BACKEND (backend), online);
+
+	if (!online) {
 		ESource *esource;
 		const gchar *display_contents = NULL;
 
 		cbgw->priv->read_only = TRUE;
 		e_cal_backend_notify_readonly (E_CAL_BACKEND (cbgw), cbgw->priv->read_only);
 
-		esource = e_cal_backend_get_source (E_CAL_BACKEND (cbgw));
+		esource = e_backend_get_source (E_BACKEND (cbgw));
 		display_contents = e_source_get_property (esource, "offline_sync");
 
 		if (!display_contents || !g_str_equal (display_contents, "1")) {
@@ -1485,27 +1487,23 @@ e_cal_backend_groupwise_remove (ECalBackendSync *backend,
 
 /* Set_mode handler for the file backend */
 static void
-e_cal_backend_groupwise_set_online (ECalBackend *backend,
-                                    gboolean is_online)
+e_cal_backend_groupwise_notify_online_cb (ECalBackend *backend,
+                                          GParamSpec *pspec)
 {
 	ECalBackendGroupwise *cbgw;
 	ECalBackendGroupwisePrivate *priv;
+	gboolean online;
 
 	cbgw = E_CAL_BACKEND_GROUPWISE (backend);
 	priv = cbgw->priv;
 
-	if ((priv->is_online ? 1 : 0) == (is_online ? 1 : 0)) {
-		e_cal_backend_notify_online (backend, priv->is_online);
-		return;
-	}
+	online = e_backend_get_online (E_BACKEND (backend));
 
 	PRIV_LOCK (priv);
 
-	priv->is_online = is_online;
-	priv->is_online_changed = TRUE;
-	if (is_online) {
+	if (online) {
 		priv->read_only = FALSE;
-		e_cal_backend_notify_online (backend, priv->is_online);
+		e_cal_backend_notify_online (backend, online);
 		e_cal_backend_notify_readonly (backend, priv->read_only);
 		if (e_cal_backend_is_opened (backend))
 			e_cal_backend_notify_auth_required (backend, TRUE, priv->credentials);
@@ -1513,7 +1511,7 @@ e_cal_backend_groupwise_set_online (ECalBackend *backend,
 		/* FIXME: make sure we update the cache before closing the connection */
 		in_offline (cbgw);
 		e_cal_backend_notify_readonly (backend, priv->read_only);
-		e_cal_backend_notify_online (backend, priv->is_online);
+		e_cal_backend_notify_online (backend, online);
 	}
 
 	PRIV_UNLOCK (priv);
@@ -1713,7 +1711,7 @@ e_cal_backend_groupwise_get_free_busy (ECalBackendSync *backend,
 
 	cbgw = E_CAL_BACKEND_GROUPWISE (backend);
 
-	if (!cbgw->priv->is_online) {
+	if (!e_backend_get_online (E_BACKEND (backend))) {
 		in_offline (cbgw);
 		g_propagate_error (perror, EDC_ERROR (RepositoryOffline));
 		return;
@@ -1751,7 +1749,7 @@ e_cal_backend_groupwise_internal_get_timezone (ECalBackend *backend,
 static EGwConnectionStatus
 update_from_server (ECalBackendGroupwise *cbgw,
                     GSList *uid_list,
-                    gchar **calobj,
+                    icalcomponent **new_icalcomp,
                     ECalComponent *comp)
 {
 	EGwConnectionStatus stat;
@@ -1778,7 +1776,6 @@ update_from_server (ECalBackendGroupwise *cbgw,
 		return stat;
 	}
 
-	comp = g_object_ref ( (ECalComponent *) list->data );
 	/* convert items into components and add them to the cache */
 	for (i = 0, tmp = list; tmp; tmp = g_list_next (tmp), i++) {
 		ECalComponent *e_cal_comp;
@@ -1790,14 +1787,11 @@ update_from_server (ECalBackendGroupwise *cbgw,
 		put_component_to_store (cbgw, e_cal_comp);
 
 		if (i == 0) {
-			*calobj = e_cal_component_get_as_string (e_cal_comp);
+			*new_icalcomp = icalcomponent_new_clone (e_cal_component_get_icalcomponent (e_cal_comp));
 		}
 
 		if (i != 0) {
-			gchar *temp;
-			temp = e_cal_component_get_as_string (e_cal_comp);
-			e_cal_backend_notify_object_created (E_CAL_BACKEND (cbgw), temp);
-			g_free (temp);
+			e_cal_backend_notify_component_created (E_CAL_BACKEND (cbgw), e_cal_component_get_icalcomponent (e_cal_comp));
 		}
 
 		g_object_unref (e_cal_comp);
@@ -1813,7 +1807,7 @@ e_cal_backend_groupwise_create_object (ECalBackendSync *backend,
                                        GCancellable *cancellable,
                                        const gchar *in_calobj,
                                        gchar **uid,
-                                       gchar **new_calobj,
+                                       icalcomponent **new_icalcomp,
                                        GError **error)
 {
 	ECalBackendGroupwise *cbgw;
@@ -1828,9 +1822,9 @@ e_cal_backend_groupwise_create_object (ECalBackendSync *backend,
 
 	e_return_data_cal_error_if_fail (E_IS_CAL_BACKEND_GROUPWISE (cbgw), InvalidArg);
 	e_return_data_cal_error_if_fail (in_calobj != NULL, InvalidArg);
-	e_return_data_cal_error_if_fail (new_calobj != NULL, InvalidArg);
+	e_return_data_cal_error_if_fail (new_icalcomp != NULL, InvalidArg);
 
-	if (!priv->is_online) {
+	if (!e_backend_get_online (E_BACKEND (backend))) {
 		in_offline (cbgw);
 		g_propagate_error (error, EDC_ERROR (RepositoryOffline));
 		return;
@@ -1853,7 +1847,7 @@ e_cal_backend_groupwise_create_object (ECalBackendSync *backend,
 	e_cal_component_set_icalcomponent (comp, icalcomp);
 
 	/* check if the object exists */
-	if (priv->is_online) {
+	if (e_backend_get_online (E_BACKEND (backend))) {
 		/* when online, send the item to the server */
 		status = e_gw_connection_create_appointment (priv->cnc, priv->container_id, cbgw, comp, &uid_list);
 
@@ -1882,7 +1876,7 @@ e_cal_backend_groupwise_create_object (ECalBackendSync *backend,
 		}
 
 		/* Get the item back from server to update the last-modified time */
-		status = update_from_server (cbgw, uid_list, new_calobj, comp);
+		status = update_from_server (cbgw, uid_list, new_icalcomp, comp);
 		if (status != E_GW_CONNECTION_STATUS_OK) {
 			g_propagate_error (error, EDC_ERROR_FAILED_STATUS (OtherError, status));
 			return;
@@ -1938,8 +1932,8 @@ e_cal_backend_groupwise_modify_object (ECalBackendSync *backend,
                                        GCancellable *cancellable,
                                        const gchar *calobj,
                                        CalObjModType mod,
-                                       gchar **old_object,
-                                       gchar **new_object,
+                                       icalcomponent **old_icalcomp,
+                                       icalcomponent **new_icalcomp,
                                        GError **error)
 {
 	ECalBackendGroupwise *cbgw;
@@ -1951,14 +1945,14 @@ e_cal_backend_groupwise_modify_object (ECalBackendSync *backend,
 	const gchar *uid = NULL;
 	gchar *rid = NULL;
 
-	*old_object = NULL;
+	*old_icalcomp = NULL;
 	cbgw = E_CAL_BACKEND_GROUPWISE (backend);
 	priv = cbgw->priv;
 
 	e_return_data_cal_error_if_fail (E_IS_CAL_BACKEND_GROUPWISE (cbgw), InvalidArg);
 	e_return_data_cal_error_if_fail (calobj != NULL, InvalidArg);
 
-	if (!priv->is_online) {
+	if (!e_backend_get_online (E_BACKEND (backend))) {
 		in_offline (cbgw);
 		g_propagate_error (error, EDC_ERROR (RepositoryOffline));
 		return;
@@ -1976,7 +1970,7 @@ e_cal_backend_groupwise_modify_object (ECalBackendSync *backend,
 	rid = e_cal_component_get_recurid_as_string (comp);
 
 	/* check if the object exists */
-	if (priv->is_online) {
+	if (e_backend_get_online (E_BACKEND (backend))) {
 		while (1) {
 			/* when online, send the item to the server */
 			cache_comp = e_cal_backend_store_get_component (priv->store, uid, rid);
@@ -2011,7 +2005,7 @@ e_cal_backend_groupwise_modify_object (ECalBackendSync *backend,
 				}
 
 				put_component_to_store (cbgw, comp);
-				*new_object = e_cal_component_get_as_string (comp);
+				*new_icalcomp = icalcomponent_new_clone (e_cal_component_get_icalcomponent (comp));
 				break;
 			}
 
@@ -2070,7 +2064,7 @@ e_cal_backend_groupwise_modify_object (ECalBackendSync *backend,
 		put_component_to_store (cbgw, comp);
 	}
 
-	*old_object = e_cal_component_get_as_string (cache_comp);
+	*old_icalcomp = icalcomponent_new_clone (e_cal_component_get_icalcomponent (cache_comp));
 	g_object_unref (cache_comp);
 	g_object_unref (comp);
 	g_free (rid);
@@ -2105,8 +2099,8 @@ e_cal_backend_groupwise_remove_object (ECalBackendSync *backend,
                                        const gchar *uid,
                                        const gchar *rid,
                                        CalObjModType mod,
-                                       gchar **old_object,
-                                       gchar **object,
+                                       icalcomponent **old_icalcomp,
+                                       icalcomponent **new_icalcomp,
                                        GError **perror)
 {
 	ECalBackendGroupwise *cbgw;
@@ -2118,10 +2112,10 @@ e_cal_backend_groupwise_remove_object (ECalBackendSync *backend,
 	cbgw = E_CAL_BACKEND_GROUPWISE (backend);
 	priv = cbgw->priv;
 
-	*old_object = *object = NULL;
+	*old_icalcomp = *new_icalcomp = NULL;
 
 	/* if online, remove the item from the server */
-	if (priv->is_online) {
+	if (e_backend_get_online (E_BACKEND (backend))) {
 		const gchar *id_to_remove = NULL;
 		icalcomponent *icalcomp;
 
@@ -2159,8 +2153,8 @@ e_cal_backend_groupwise_remove_object (ECalBackendSync *backend,
 					g_propagate_error (perror, EDC_ERROR (ObjectNotFound));
 					return;
 				}
-				*object = NULL;
-				*old_object = strdup (calobj);
+				*new_icalcomp = NULL;
+				*old_icalcomp = icalparser_parse_string (calobj);
 				g_free (calobj);
 				return;
 			} else {
@@ -2215,8 +2209,8 @@ e_cal_backend_groupwise_remove_object (ECalBackendSync *backend,
 				/* Setting NULL would trigger another signal.
 				 * We do not set the *object to NULL  */
 				g_slist_free (comp_list);
-				*old_object = strdup (calobj);
-				*object = NULL;
+				*old_icalcomp = icalparser_parse_string (calobj);
+				*new_icalcomp = NULL;
 				g_free (calobj);
 				return;
 			} else {
@@ -2244,18 +2238,15 @@ fetch_attachments (ECalBackendGroupwise *cbgw,
 	GSList *attach_list = NULL, *new_attach_list = NULL;
 	GSList *l;
 	gchar *dest_url, *dest_file;
-	gint fd;
-	const gchar *cache_dir;
+	gint fd, fileindex;
 	const gchar *uid;
 
 	e_cal_component_get_attachment_list (comp, &attach_list);
 	e_cal_component_get_uid (comp, &uid);
-	/*FIXME  get the uri rather than computing the path */
-	cache_dir = e_cal_backend_get_cache_dir (E_CAL_BACKEND (cbgw));
 
-	for (l = attach_list; l; l = l->next) {
+	for (l = attach_list, fileindex = 0; l; l = l->next, fileindex++) {
 		gchar *sfname = (gchar *) l->data;
-		gchar *filename, *new_filename;
+		gchar *filename;
 		GMappedFile *mapped_file;
 		GError *error = NULL;
 
@@ -2265,10 +2256,8 @@ fetch_attachments (ECalBackendGroupwise *cbgw,
 			continue;
 		}
 		filename = g_path_get_basename (sfname);
-		new_filename = g_strconcat (uid, "-", filename, NULL);
+		dest_file = e_cal_backend_create_cache_filename (E_CAL_BACKEND (cbgw), uid, filename, fileindex);
 		g_free (filename);
-		dest_file = g_build_filename (cache_dir, new_filename, NULL);
-		g_free (new_filename);
 		fd = g_open (dest_file, O_RDWR | O_CREAT | O_TRUNC | O_BINARY, 0600);
 		if (fd == -1) {
 			/* TODO handle error conditions */
@@ -2485,15 +2474,13 @@ e_cal_backend_groupwise_receive_objects (ECalBackendSync *backend,
                                          GError **perror)
 {
 	ECalBackendGroupwise *cbgw;
-	ECalBackendGroupwisePrivate *priv;
 	icalcomponent *icalcomp, *subcomp;
 	icalcomponent_kind kind;
 	GError *err = NULL;
 
 	cbgw = E_CAL_BACKEND_GROUPWISE (backend);
-	priv = cbgw->priv;
 
-	if (!priv->is_online) {
+	if (!e_backend_get_online (E_BACKEND (backend))) {
 		in_offline (cbgw);
 		g_propagate_error (perror, EDC_ERROR (RepositoryOffline));
 		return;
@@ -2557,7 +2544,7 @@ send_object (ECalBackendGroupwise *cbgw,
 		return;
 	}
 
-	if (priv->is_online) {
+	if (e_backend_get_online (E_BACKEND (cbgw))) {
 		if (method == ICAL_METHOD_CANCEL) {
 			const gchar *retract_comment = NULL;
 			gboolean all_instances = FALSE;
@@ -2598,16 +2585,14 @@ e_cal_backend_groupwise_send_objects (ECalBackendSync *backend,
 	icalcomponent_kind kind;
 	icalproperty_method method;
 	ECalBackendGroupwise *cbgw;
-	ECalBackendGroupwisePrivate *priv;
 	GError *err = NULL;
 
 	*users = NULL;
 	*modified_calobj = NULL;
 
 	cbgw = E_CAL_BACKEND_GROUPWISE (backend);
-	priv = cbgw->priv;
 
-	if (!priv->is_online) {
+	if (!e_backend_get_online (E_BACKEND (backend))) {
 		in_offline (cbgw);
 		g_propagate_error (perror, EDC_ERROR (RepositoryOffline));
 		return;
@@ -2687,6 +2672,10 @@ e_cal_backend_groupwise_init (ECalBackendGroupwise *cbgw)
 	cbgw->priv = priv;
 
 	e_cal_backend_sync_set_lock (E_CAL_BACKEND_SYNC (cbgw), TRUE);
+
+	g_signal_connect (
+		cbgw, "notify::online",
+		G_CALLBACK (e_cal_backend_groupwise_notify_online_cb), NULL);
 }
 
 /* Class initialization function for the gw backend */
@@ -2721,7 +2710,6 @@ e_cal_backend_groupwise_class_init (ECalBackendGroupwiseClass *class)
 	sync_class->get_free_busy_sync		= e_cal_backend_groupwise_get_free_busy;
 
 	backend_class->start_view		= e_cal_backend_groupwise_start_view;
-	backend_class->set_online		= e_cal_backend_groupwise_set_online;
 	backend_class->internal_get_timezone	= e_cal_backend_groupwise_internal_get_timezone;
 }
 
