@@ -112,8 +112,9 @@ CamelServiceAuthType camel_groupwise_password_authtype = {
 };
 
 static gint
-groupwise_auto_detect_cb (CamelURL *url, GHashTable **auto_detected,
-			 GError **error)
+groupwise_auto_detect_cb (CamelURL *url,
+                          GHashTable **auto_detected,
+                          GError **error)
 {
 	*auto_detected = g_hash_table_new (g_str_hash, g_str_equal);
 
@@ -149,7 +150,8 @@ camel_provider_module_init (void)
 }
 
 static void
-add_hash (guint *hash, gchar *s)
+add_hash (guint *hash,
+          gchar *s)
 {
 	if (s)
 		*hash ^= g_str_hash(s);
@@ -169,7 +171,8 @@ groupwise_url_hash (gconstpointer key)
 }
 
 static gint
-check_equal (gchar *s1, gchar *s2)
+check_equal (gchar *s1,
+             gchar *s2)
 {
 	if (s1 == NULL) {
 		if (s2 == NULL)
@@ -185,7 +188,8 @@ check_equal (gchar *s1, gchar *s2)
 }
 
 static gint
-groupwise_url_equal (gconstpointer a, gconstpointer b)
+groupwise_url_equal (gconstpointer a,
+                     gconstpointer b)
 {
 	const CamelURL *u1 = a, *u2 = b;
 
