@@ -61,15 +61,13 @@ struct _CamelGroupwiseStorePrivate {
 	gchar *server_name;
 	gchar *user;
 
-	gchar *base_url;
-
-	GHashTable *id_hash; /*get names from ids*/
-	GHashTable *name_hash;/*get ids from names*/
+	GHashTable *id_hash; /*get names from ids */
+	GHashTable *name_hash;/*get ids from names */
 	GHashTable *parent_hash;
 	EGwConnection *cnc;
 };
 
-extern CamelServiceAuthType camel_groupwise_password_authtype; /*for the query_auth_types function*/
+extern CamelServiceAuthType camel_groupwise_password_authtype; /*for the query_auth_types function */
 
 static GInitableIface *parent_initable_interface;
 
@@ -90,98 +88,14 @@ groupwise_hash_folder_name (gconstpointer key)
 }
 
 static gint
-groupwise_compare_folder_name (gconstpointer a, gconstpointer b)
+groupwise_compare_folder_name (gconstpointer a,
+                               gconstpointer b)
 {
 	gconstpointer aname = a, bname = b;
 
 	return g_str_equal (aname, bname);
 }
 
-static gboolean
-groupwise_auth_loop (CamelService *service,
-                     GCancellable *cancellable,
-                     GError **error)
-{
-	CamelSession *session = camel_service_get_session (service);
-	CamelStore *store = CAMEL_STORE (service);
-	CamelSettings *settings;
-	CamelGroupwiseStore *groupwise_store = CAMEL_GROUPWISE_STORE (store);
-	CamelGroupwiseStorePrivate *priv = groupwise_store->priv;
-	CamelNetworkSecurityMethod security_method;
-	CamelURL *url;
-	gboolean authenticated = FALSE;
-	gchar *uri;
-	guint32 prompt_flags = CAMEL_SESSION_PASSWORD_SECRET;
-	const gchar *scheme;
-	EGwConnectionErrors errors = {E_GW_CONNECTION_STATUS_INVALID_OBJECT, NULL};
-	guint16 soap_port;
-
-	url = camel_service_get_camel_url (service);
-	settings = camel_service_get_settings (service);
-
-	security_method = camel_network_settings_get_security_method (
-		CAMEL_NETWORK_SETTINGS (settings));
-
-	soap_port = camel_groupwise_settings_get_soap_port (
-		CAMEL_GROUPWISE_SETTINGS (settings));
-
-	if (security_method == CAMEL_NETWORK_SECURITY_METHOD_NONE)
-		scheme = "http";
-	else
-		scheme = "https";
-
-	uri = g_strdup_printf (
-		"%s://%s:%u/soap", scheme, priv->server_name, soap_port);
-
-	url->passwd = NULL;
-
-	while (!authenticated) {
-
-		if (!url->passwd && !(store->flags & CAMEL_STORE_PROXY)) {
-			gchar *prompt;
-
-			prompt = camel_session_build_password_prompt (
-				"GroupWise", url->user, url->host);
-			url->passwd =
-				camel_session_get_password (
-					session, service, prompt,
-					"password", prompt_flags, error);
-			g_free (prompt);
-
-			if (!url->passwd) {
-				g_set_error (
-					error, G_IO_ERROR,
-					G_IO_ERROR_CANCELLED,
-					_("You did not enter a password."));
-				return FALSE;
-			}
-		}
-
-		priv->cnc = e_gw_connection_new_with_error_handler (uri, priv->user, url->passwd, &errors);
-
-		if (!E_IS_GW_CONNECTION (priv->cnc)) {
-			if (errors.status == E_GW_CONNECTION_STATUS_INVALID_PASSWORD) {
-				/* We need to un-cache the password before prompting again */
-				prompt_flags |= CAMEL_SESSION_PASSWORD_REPROMPT;
-				g_free (url->passwd);
-				url->passwd = NULL;
-			} else {
-				g_set_error (
-					error, CAMEL_SERVICE_ERROR,
-					CAMEL_SERVICE_ERROR_UNAVAILABLE,
-					"%s", errors.description ?
-					errors.description :
-					_("You must be working online to complete this operation"));
-				return FALSE;
-			}
-		} else
-			authenticated = TRUE;
-
-	}
-
-	return TRUE;
-}
-
 static gboolean
 check_for_connection (CamelService *service,
                       GCancellable *cancellable,
@@ -201,14 +115,14 @@ check_for_connection (CamelService *service,
 	ai = camel_getaddrinfo(priv->server_name, "groupwise", &hints, cancellable, &local_error);
 	if (ai == NULL && g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
 		gchar *port_string;
-		guint16 soap_port;
+		const gchar *soap_port;
 
 		g_clear_error (&local_error);
 
 		soap_port = camel_groupwise_settings_get_soap_port (
 			CAMEL_GROUPWISE_SETTINGS (settings));
 
-		port_string = g_strdup_printf ("%u", soap_port);
+		port_string = g_strdup(soap_port);
 		ai = camel_getaddrinfo (
 			priv->server_name, port_string, &hints,
 			cancellable, &local_error);
@@ -228,7 +142,8 @@ check_for_connection (CamelService *service,
 
 /* resets the current folder. To just free current folder, pass NULL for folder */
 void
-groupwise_store_set_current_folder (CamelGroupwiseStore *groupwise_store, CamelFolder *folder)
+groupwise_store_set_current_folder (CamelGroupwiseStore *groupwise_store,
+                                    CamelFolder *folder)
 {
 
 	camel_service_lock (CAMEL_SERVICE (groupwise_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
@@ -272,7 +187,13 @@ groupwise_connect_sync (CamelService *service,
 		return TRUE;
 	}
 
-	if (!check_for_connection (service, cancellable, error) || !groupwise_auth_loop (service, cancellable, error)) {
+	if (!check_for_connection (service, cancellable, error)) {
+		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
+		camel_service_disconnect_sync (service, TRUE, NULL);
+		return FALSE;
+	}
+
+	if (!camel_session_authenticate_sync (session, service, NULL, cancellable, error)) {
 		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 		camel_service_disconnect_sync (service, TRUE, NULL);
 		return FALSE;
@@ -310,7 +231,9 @@ groupwise_connect_sync (CamelService *service,
 }
 #if 0
 static void
-groupwise_disconnect_cleanup (CamelService *service, gboolean clean, GError **error)
+groupwise_disconnect_cleanup (CamelService *service,
+                              gboolean clean,
+                              GError **error)
 {
 	CamelGroupwiseStore *groupwise_store = CAMEL_GROUPWISE_STORE (service);
 	CamelGroupwiseStorePrivate *priv = groupwise_store->priv;
@@ -337,10 +260,6 @@ groupwise_disconnect_cleanup (CamelService *service, gboolean clean, GError **er
 			g_free (priv->use_ssl);
 			priv->use_ssl = NULL;
 		}
-		if (priv->base_url) {
-			g_free (priv->base_url);
-			priv->base_url = NULL;
-		}
 
 		if (groupwise_store->root_container)
 			g_free (groupwise_store->root_container);
@@ -384,7 +303,7 @@ groupwise_disconnect_sync (CamelService *service,
 	return TRUE;
 }
 
-static  GList*
+static  GList *
 groupwise_store_query_auth_types_sync (CamelService *service,
                                        GCancellable *cancellable,
                                        GError **error)
@@ -412,7 +331,9 @@ groupwise_is_system_folder (const gchar *folder_name)
 
 /*Build/populate CamelFolderInfo structure based on the imap_build_folder_info function*/
 static CamelFolderInfo *
-groupwise_build_folder_info (CamelGroupwiseStore *gw_store, const gchar *parent_name, const gchar *folder_name)
+groupwise_build_folder_info (CamelGroupwiseStore *gw_store,
+                             const gchar *parent_name,
+                             const gchar *folder_name)
 {
 	const gchar *name;
 	CamelFolderInfo *fi;
@@ -452,7 +373,9 @@ groupwise_build_folder_info (CamelGroupwiseStore *gw_store, const gchar *parent_
 }
 
 static void
-groupwise_forget_folder (CamelGroupwiseStore *gw_store, const gchar *folder_name, GError **error)
+groupwise_forget_folder (CamelGroupwiseStore *gw_store,
+                         const gchar *folder_name,
+                         GError **error)
 {
 	CamelService *service;
 	const gchar *user_data_dir;
@@ -626,7 +549,7 @@ groupwise_store_get_folder_sync (CamelStore *store,
 			cancellable,
 			_("Fetching summary information for new messages in %s"),
 			camel_folder_get_display_name (folder));
-		camel_folder_summary_clear (folder->summary);
+		camel_folder_summary_clear (folder->summary, NULL);
 
 		while (!done) {
 				status = e_gw_connection_read_cursor (priv->cnc, container_id,
@@ -648,7 +571,7 @@ groupwise_store_get_folder_sync (CamelStore *store,
 						count = total;
 
 					camel_operation_progress (
-						cancellable, (100*count)/total);
+						cancellable, (100 * count) / total);
 				}
 
 				gw_update_summary (folder, list, cancellable, error);
@@ -716,7 +639,7 @@ gw_store_reload_folder (CamelGroupwiseStore *gw_store,
 	}
 
 	if (!E_IS_GW_CONNECTION ( priv->cnc)) {
-		if (!groupwise_connect_sync (CAMEL_SERVICE ((CamelStore*) gw_store), cancellable, error)) {
+		if (!groupwise_connect_sync (CAMEL_SERVICE ((CamelStore *) gw_store), cancellable, error)) {
 			camel_service_unlock (CAMEL_SERVICE (gw_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 			return FALSE;
 		}
@@ -731,7 +654,7 @@ gw_store_reload_folder (CamelGroupwiseStore *gw_store,
 	}
 
 	summary = (CamelGroupwiseSummary *) folder->summary;
-	camel_folder_summary_clear (folder->summary);
+	camel_folder_summary_clear (folder->summary, NULL);
 	camel_folder_summary_save_to_db (folder->summary, NULL);
 
 	summary_count = camel_folder_summary_count (folder->summary);
@@ -775,7 +698,7 @@ gw_store_reload_folder (CamelGroupwiseStore *gw_store,
 							return FALSE;
 					}
 					/* This full block is repeated in the get_folder code as well. We need
-					   better modularity */
+					 * better modularity */
 					count += CURSOR_ITEM_LIMIT;
 
 					if (total > 0) {
@@ -791,8 +714,8 @@ gw_store_reload_folder (CamelGroupwiseStore *gw_store,
 					gw_update_summary (folder, list, cancellable, error);
 
 					/* For shared-folders created by the user, we don't get the total number of messages,
-					   in the getFolderList call. So, we need to wait until an empty list is returned in the
-					   read cursor call. Hence, we need the !list checking in the code below */
+					 * in the getFolderList call. So, we need to wait until an empty list is returned in the
+					 * read cursor call. Hence, we need the !list checking in the code below */
 					if (count == total || !list)
 							done = TRUE;
 
@@ -859,9 +782,9 @@ convert_to_folder_info (CamelGroupwiseStore *store,
 		(type == E_GW_CONTAINER_TYPE_TRASH) )
 		fi->flags |= CAMEL_FOLDER_SYSTEM;
 	/*
-	   parent_hash contains the "parent id <-> container id" combination. So we form
-	   the path for the full name in camelfolder info by looking up the hash table until
-	   NULL is found
+	 * parent_hash contains the "parent id <-> container id" combination. So we form
+	 * the path for the full name in camelfolder info by looking up the hash table until
+	 * NULL is found
 	 */
 
 	parent = e_gw_container_get_parent_id (container);
@@ -922,7 +845,9 @@ convert_to_folder_info (CamelGroupwiseStore *store,
 }
 
 static void
-get_folders_free (gpointer k, gpointer v, gpointer d)
+get_folders_free (gpointer k,
+                  gpointer v,
+                  gpointer d)
 {
 	CamelFolderInfo *fi = v;
 	camel_folder_info_free (fi);
@@ -953,7 +878,7 @@ groupwise_folders_sync (CamelGroupwiseStore *store,
 	list = folder_list;
 
 	/*populate the hash table for finding the mapping from container id <-> folder name*/
-	for (;temp_list != NULL; temp_list = g_list_next (temp_list) ) {
+	for (; temp_list != NULL; temp_list = g_list_next (temp_list) ) {
 		const gchar *name, *id, *parent;
 		name = e_gw_container_get_name (E_GW_CONTAINER (temp_list->data));
 		id = e_gw_container_get_id (E_GW_CONTAINER (temp_list->data));
@@ -974,7 +899,7 @@ groupwise_folders_sync (CamelGroupwiseStore *store,
 
 	present = g_hash_table_new (g_str_hash, g_str_equal);
 
-	for (;folder_list != NULL; folder_list = g_list_next (folder_list)) {
+	for (; folder_list != NULL; folder_list = g_list_next (folder_list)) {
 		EGwContainerType type;
 		EGwContainer *container = E_GW_CONTAINER (folder_list->data);
 
@@ -1001,7 +926,7 @@ groupwise_folders_sync (CamelGroupwiseStore *store,
 	count = camel_store_summary_count ((CamelStoreSummary *) store->summary);
 
 	count = camel_store_summary_count ((CamelStoreSummary *) store->summary);
-	for (i=0;i<count;i++) {
+	for (i = 0; i < count; i++) {
 		si = camel_store_summary_index ((CamelStoreSummary *) store->summary, i);
 		if (si == NULL)
 			continue;
@@ -1024,8 +949,10 @@ groupwise_folders_sync (CamelGroupwiseStore *store,
 }
 
 static CamelFolderInfo *
-groupwise_get_folder_info_offline (CamelStore *store, const gchar *top,
-			 guint32 flags, GError **error)
+groupwise_get_folder_info_offline (CamelStore *store,
+                                   const gchar *top,
+                                   guint32 flags,
+                                   GError **error)
 {
 	CamelGroupwiseStore *groupwise_store = CAMEL_GROUPWISE_STORE (store);
 	CamelFolderInfo *fi;
@@ -1049,7 +976,7 @@ groupwise_get_folder_info_offline (CamelStore *store, const gchar *top,
 
 	path = gw_concat (name, "*");
 
-	for (i=0;i<camel_store_summary_count ((CamelStoreSummary *) groupwise_store->summary);i++) {
+	for (i = 0; i < camel_store_summary_count ((CamelStoreSummary *) groupwise_store->summary); i++) {
 		CamelStoreInfo *si = camel_store_summary_index ((CamelStoreSummary *) groupwise_store->summary, i);
 
 		if (si == NULL)
@@ -1084,7 +1011,7 @@ groupwise_store_get_folder_info_sync (CamelStore *store,
 	CamelFolderInfo *info = NULL;
 
 	/* Do not call groupwise_store_connected function as it would internall call folders_sync
-	   to populate the hash table which is used for mapping container id */
+	 * to populate the hash table which is used for mapping container id */
 	if (!(camel_offline_store_get_online (CAMEL_OFFLINE_STORE (store))
 	    && camel_service_connect_sync ((CamelService *) store, error)))
 		goto offline;
@@ -1143,7 +1070,7 @@ create_junk_folder (CamelStore *store)
 	return root;
 }
 
-static CamelFolderInfo*
+static CamelFolderInfo *
 groupwise_store_create_folder_sync (CamelStore *store,
                                     const gchar *parent_name,
                                     const gchar *folder_name,
@@ -1325,7 +1252,8 @@ groupwise_store_rename_folder_sync (CamelStore *store,
 }
 
 gchar *
-groupwise_get_name (CamelService *service, gboolean brief)
+groupwise_get_name (CamelService *service,
+                    gboolean brief)
 {
 	CamelURL *url;
 
@@ -1341,7 +1269,8 @@ groupwise_get_name (CamelService *service, gboolean brief)
 }
 
 const gchar *
-camel_groupwise_store_container_id_lookup (CamelGroupwiseStore *gw_store, const gchar *folder_name)
+camel_groupwise_store_container_id_lookup (CamelGroupwiseStore *gw_store,
+                                           const gchar *folder_name)
 {
 	CamelGroupwiseStorePrivate *priv = gw_store->priv;
 
@@ -1349,7 +1278,8 @@ camel_groupwise_store_container_id_lookup (CamelGroupwiseStore *gw_store, const
 }
 
 const gchar *
-camel_groupwise_store_folder_lookup (CamelGroupwiseStore *gw_store, const gchar *container_id)
+camel_groupwise_store_folder_lookup (CamelGroupwiseStore *gw_store,
+                                     const gchar *container_id)
 {
 	CamelGroupwiseStorePrivate *priv = gw_store->priv;
 
@@ -1362,12 +1292,6 @@ cnc_lookup (CamelGroupwiseStorePrivate *priv)
 	return priv->cnc;
 }
 
-const gchar *
-groupwise_base_url_lookup (CamelGroupwiseStorePrivate *priv)
-{
-	return priv->base_url;
-}
-
 static CamelFolder *
 groupwise_store_get_trash_folder_sync (CamelStore *store,
                                        GCancellable *cancellable,
@@ -1441,7 +1365,8 @@ camel_groupwise_store_connected (CamelGroupwiseStore *store,
 }
 
 static gint
-match_path (const gchar *path, const gchar *name)
+match_path (const gchar *path,
+            const gchar *name)
 {
 	gchar p, n;
 
@@ -1498,7 +1423,6 @@ groupwise_store_finalize (GObject *object)
 
 	g_free (groupwise_store->priv->user);
 	g_free (groupwise_store->priv->server_name);
-	g_free (groupwise_store->priv->base_url);
 	g_free (groupwise_store->root_container);
 
 	if (groupwise_store->priv->id_hash != NULL)
@@ -1514,6 +1438,72 @@ groupwise_store_finalize (GObject *object)
 	G_OBJECT_CLASS (camel_groupwise_store_parent_class)->finalize (object);
 }
 
+static CamelAuthenticationResult
+groupwise_store_authenticate_sync (CamelService *service,
+                                   const gchar *mechanism,
+                                   GCancellable *cancellable,
+                                   GError **error)
+{
+	CamelStore *store = CAMEL_STORE (service);
+	CamelSettings *settings;
+	CamelGroupwiseStore *groupwise_store = CAMEL_GROUPWISE_STORE (store);
+	CamelGroupwiseStorePrivate *priv = groupwise_store->priv;
+	CamelNetworkSecurityMethod security_method;
+	CamelAuthenticationResult result;
+	gchar *uri;
+	const gchar *scheme;
+	const gchar *password;
+	EGwConnectionErrors errors = {E_GW_CONNECTION_STATUS_INVALID_OBJECT, NULL};
+	const gchar *soap_port;
+
+	password = camel_service_get_password (service);
+	settings = camel_service_get_settings (service);
+
+	if (password == NULL) {
+		g_set_error_literal (
+			error, CAMEL_SERVICE_ERROR,
+			CAMEL_SERVICE_ERROR_CANT_AUTHENTICATE,
+			_("Authentication password not available"));
+		return CAMEL_AUTHENTICATION_ERROR;
+	}
+
+	security_method = camel_network_settings_get_security_method (
+		CAMEL_NETWORK_SETTINGS (settings));
+
+	soap_port = camel_groupwise_settings_get_soap_port (
+		CAMEL_GROUPWISE_SETTINGS (settings));
+
+	if (security_method == CAMEL_NETWORK_SECURITY_METHOD_NONE)
+		scheme = "http";
+	else
+		scheme = "https";
+
+	uri = g_strdup_printf (
+		"%s://%s:%s/soap", scheme, priv->server_name, soap_port);
+
+	priv->cnc = e_gw_connection_new_with_error_handler (
+		uri, priv->user, password, &errors);
+
+	g_free (uri);
+
+	if (E_IS_GW_CONNECTION (priv->cnc)) {
+		result = CAMEL_AUTHENTICATION_ACCEPTED;
+	} else if (errors.status == E_GW_CONNECTION_STATUS_INVALID_PASSWORD) {
+		result = CAMEL_AUTHENTICATION_REJECTED;
+	} else {
+		g_set_error (
+			error, CAMEL_SERVICE_ERROR,
+			CAMEL_SERVICE_ERROR_UNAVAILABLE,
+			"%s", errors.description ?
+			errors.description :
+			_("You must be working online "
+			  "to complete this operation"));
+		result = CAMEL_AUTHENTICATION_ERROR;
+	}
+
+	return result;
+}
+
 static gboolean
 groupwise_store_initable_init (GInitable *initable,
                                GCancellable *cancellable,
@@ -1560,11 +1550,6 @@ groupwise_store_initable_init (GInitable *initable,
 	groupwise_store->priv->server_name = g_strdup (url->host);
 	groupwise_store->priv->user = g_strdup (url->user);
 
-	/*base url*/
-	groupwise_store->priv->base_url = camel_url_to_string (
-		url, CAMEL_URL_HIDE_PASSWORD |
-		CAMEL_URL_HIDE_PARAMS | CAMEL_URL_HIDE_AUTH);
-
 	/*Hash Table*/
 	groupwise_store->priv->id_hash =
 		g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
@@ -1615,6 +1600,7 @@ camel_groupwise_store_class_init (CamelGroupwiseStoreClass *class)
 	service_class->get_name = groupwise_get_name;
 	service_class->connect_sync = groupwise_connect_sync;
 	service_class->disconnect_sync = groupwise_disconnect_sync;
+	service_class->authenticate_sync = groupwise_store_authenticate_sync;
 	service_class->query_auth_types_sync = groupwise_store_query_auth_types_sync;
 
 	store_class = CAMEL_STORE_CLASS (class);
