@@ -50,7 +50,9 @@ struct AcceptData {
 void org_gnome_popup_wizard (EPlugin *ep, EMEventTargetMessage *target);
 
 static void
-install_folder_response (EMFolderSelector *emfs, gint response, gpointer *data)
+install_folder_response (EMFolderSelector *emfs,
+                         gint response,
+                         gpointer *data)
 {
 	struct AcceptData *accept_data = (struct AcceptData *) data;
 	EMFolderTreeModel *model;
@@ -159,7 +161,8 @@ accept_free (gpointer data)
 }
 
 static void
-apply_clicked (GtkAssistant *assistant, CamelMimeMessage *msg)
+apply_clicked (GtkAssistant *assistant,
+               CamelMimeMessage *msg)
 {
 	EShell *shell;
 	EShellBackend *shell_backend;
@@ -196,7 +199,8 @@ apply_clicked (GtkAssistant *assistant, CamelMimeMessage *msg)
 }
 
 void
-org_gnome_popup_wizard (EPlugin *ep, EMEventTargetMessage *target)
+org_gnome_popup_wizard (EPlugin *ep,
+                        EMEventTargetMessage *target)
 {
 	CamelInternetAddress *from_addr = NULL;
 	const gchar *name;
