@@ -475,7 +475,7 @@ proxy_get_cnc (EAccount *account,
 	const gchar *poa_address;
 	const gchar *scheme;
 	gboolean remember;
-	guint16 soap_port;
+	gchar *soap_port;
 
 	url = camel_url_new (account->source->url, NULL);
 	if (url == NULL)
@@ -502,7 +502,7 @@ proxy_get_cnc (EAccount *account,
 
 	key =  g_strdup_printf ("groupwise://%s@%s/", url->user, poa_address);
 
-	uri = g_strdup_printf ("%s://%s:%u/soap", scheme, poa_address, soap_port);
+	uri = g_strdup_printf ("%s://%s:%s/soap", scheme, poa_address, soap_port);
 
 	failed_auth = "";
 	cnc = NULL;
